﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Controls;
using System.Diagnostics;

namespace Test_Theme.classes
{
    public class DragManager
    {
        private Point _startMousePosition;
        private bool _isDragging = false;
        private UIElement _layoutObj = null;
        private UIElement _dragObj = null;

        //properties
        public bool IsCanGetCollisionMove { get; set; }
        public bool IsCanGetCollisionUp { get; set; }

        //event
        public event EventHandler<CollisionEventArgs> CollisionMove;
        public event EventHandler<CollisionEventArgs> CollisionUp;

        //constructor
        public DragManager(UIElement layoutObj)
        {
            this._layoutObj = layoutObj;
            IsCanGetCollisionMove = false;
            IsCanGetCollisionUp = false;
        }

        #region || method ||
        public void EnableDragableElement(UIElement dragObj)
        {
            this._dragObj = dragObj;
            this._dragObj.MouseLeftButtonDown += element_MouseLeftButtonDown;
            this._dragObj.MouseMove += elementToDrag_MouseMove;
            this._dragObj.MouseLeftButtonUp += elementToDrag_MouseLeftButtonUp;
        }
        public void DisableDragableElement()
        {
            _dragObj.MouseLeftButtonDown -= element_MouseLeftButtonDown;
            _dragObj.MouseMove -= elementToDrag_MouseMove;
            _dragObj.MouseLeftButtonUp -= elementToDrag_MouseLeftButtonUp;
        }

        //set margin
        private void setMargin(UIElement element, Point curPos)
        {
            FrameworkElement gridDragObj = (element as FrameworkElement);
            if (gridDragObj == null)
            {
                Debug.WriteLine("UserControl Convert error");
                return;
            }
            gridDragObj.Margin = new Thickness(curPos.X - _startMousePosition.X, curPos.Y - _startMousePosition.Y, 0, 0);
        }
        //find elements
        private List<UIElement> findElement(UIElement element, Point curPos)
        {
            List<UIElement> collidedElements = VisualTreeHelper.FindElementsInHostCoordinates(curPos, _layoutObj) as List<UIElement>;
            collidedElements.Remove(element);
            collidedElements.Remove(_layoutObj);
            return collidedElements;
        }
        #endregion

        #region || handler ||
        void elementToDrag_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (_isDragging)
            {
                UIElement element = (UIElement)sender;
                Point currentMousePosition = e.GetPosition(_layoutObj);

                List<UIElement> collidedElements = null;
                if (IsCanGetCollisionMove)
                {
                    collidedElements = findElement(element, currentMousePosition);
                }

                if (CollisionMove != null)
                {
                    CollisionEventArgs args = new CollisionEventArgs()
                    {
                        Element = element,
                        Position = currentMousePosition,
                        PositionDragElement = new Point((element as FrameworkElement).Margin.Left, (element as FrameworkElement).Margin.Top),
                        CollidedElements = collidedElements
                    };
                    CollisionMove(this, args);
                }

                ((UIElement)sender).ReleaseMouseCapture();
                _isDragging = false;
            }
        }

        void elementToDrag_MouseMove(object sender, MouseEventArgs e)
        {
            if (_isDragging)
            {
                UIElement element = (UIElement)sender;
                Point currentMousePosition = e.GetPosition(_layoutObj);

                setMargin(element, currentMousePosition);

                List<UIElement> collidedElements = null;
                if (IsCanGetCollisionUp)
                {
                    collidedElements = findElement(element, currentMousePosition);
                }
                
                if (CollisionMove != null)
                {
                    CollisionEventArgs args = new CollisionEventArgs()
                    {
                        Element = element,
                        Position = currentMousePosition,
                        PositionDragElement = new Point((element as FrameworkElement).Margin.Left, (element as FrameworkElement).Margin.Top),
                        CollidedElements = collidedElements
                    };
                    CollisionMove(this, args);
                }
            }
        }

        void element_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;
            _isDragging = true;
            _startMousePosition = e.GetPosition((UIElement)sender);
            ((UIElement)sender).CaptureMouse();
        }
        #endregion
    }

    #region || event argument class ||
    public class CollisionEventArgs : EventArgs
    {
        //drag object
        public UIElement Element { get; set; }
        //mouse point
        public Point Position { get; set; }
        //drag object margin value
        public Point PositionDragElement { get; set; }
        //CollidedElements
        public List<UIElement> CollidedElements { get; set; }
    }
    #endregion
}
